@@ -875,7 +875,7 @@ class tube(Timeout, Logger):
 
         go = threading.Event()
         def recv_thread():
-            while not go.isSet():
+            while not go.is_set():
                 try:
                     cur = self.recv(timeout = 0.05)
                     cur = cur.replace(self.newline, b'\n')
@@ -897,11 +897,11 @@ class tube(Timeout, Logger):
         try:
             os_linesep = os.linesep.encode()
             to_skip = b''
-            while not go.isSet():
+            while not go.is_set():
                 if term.term_mode:
                     data = term.readline.readline(prompt = prompt, float = True)
-                    if data:
-                        data += self.newline
+                    if data.endswith(b'\n') and self.newline != b'\n':
+                        data = data[:-1] + self.newline
                 else:
                     stdin = getattr(sys.stdin, 'buffer', sys.stdin)
                     data = stdin.read(1)
